(* Copyright (c) 2002 INRIA *)
(* Author : Gaurav Chanda , Projet CRISTAL *)

(** This module gives the description of the [base64Binary_s] type. *)

(** This is the lower bound for the [minLength] attribute *)
value base64Binary_min : int ;

(** This is the upper bound for the [maxLength] attribute *)
value base64Binary_max : int ;

(** This gives the general description of the [base64binary_s_sig] class *)
class type base64Binary_s_sig = 
        object
             
             value mutable name : string ;
             (** This contains the name of the XML-element concerned *)
             
             value mutable v : string ;
             (** This contains the value of the object *)
             
             value mutable length : int ;
             (** This contains information about the length of the string, if
             fixed. Otherwise, it is set to -1 *)
             
             value mutable minLength : int ;
             (** This contains information about the minimum length of the string, if
             fixed. Otherwise, it is set to -1 *)
             
             value mutable maxLength : int ;
             (** This contains information about the maximum length of the string, if
             fixed. Otherwise, it is set to -1 *)
             
             value mutable is_enum : bool ;
             (** This indicates whether the type is an enumerated type *)
             
             value mutable is_fixed : bool ;
             (** This indicates whether the [length] field of this object can be
             altered by a derived class *)

             value mutable is_fixed_minl : bool ;
             (** This indicates whether the [minLength] field of this object can be
             altered by a derived class *)

             value mutable is_fixed_maxl : bool ;
             (** This indicates whether the [maxLength] field of this object can be
             altered by a derived class *)

             value mutable enumlist : list string ;
             (** This contains the list of entries, required to define the
             enumerated datatype *)
             
             value mutable whitespace : string ;
             (** This contains the information on how to handle white spaces *)

             method get_val : unit -> string ;
             (** This function returns the value held by the object *)
             
             method set_obj : string -> unit ;
             (** Sets the value of the object with the given string after making
             other attribute checks *)
             
             method to_string : unit -> string ;
             (** Returns the XML document corresponding to this object *)
             
             method set_struct : string -> Pxp_document.node (Pxp_document.extension (Pxp_document.node 'a)) -> unit ;
             (** Takes as argument, an element name and the schema that
             specifies the various attributes that need to be fixed *)

        end ;

(** An instance of the [base64Binary_s_sig] class *)
class base64Binary_s : base64Binary_s_sig ;

