(* Copyright (c) 2002 INRIA *)
(* Author : Gaurav Chanda , Projet CRISTAL *)

open Pxp_yacc ;
open Pxp_document ;
open Pxp_types ;
open Utils ;

class type boolean_s_sig = 
        object
             
             value mutable name : string ;
             value mutable v : bool ;
             value mutable whitespace : string ;

             method get_val : unit -> bool ;
             method set_obj : string -> unit ;
             method to_string : unit -> string ;
             method set_struct : string -> Pxp_document.node (Pxp_document.extension (Pxp_document.node 'a)) -> unit ;

        end ;

class boolean_s : boolean_s_sig =
        object(self)

                value mutable name = "Not_Named" ;
                value mutable v = False ;
                value mutable whitespace = "collapsed" ;
                
                method get_val () = v ;

                method private preserve str = str ;

                method private rf ch = match ch with
                  [ '\t' -> ' '
                  | '\n' -> ' '
                  | '\r' -> ' '
                  | c -> c
                  ];
                  
                method private replace str = implode (zip self#rf (explode str));

                method  private cf ls = 
                     let l = ref ls in
                     let b = (l.val = []) in
                     let _ = if (not b) then while (List.hd l.val = ' ') do 
                             {
                                     l.val := List.tl l.val 
                             } 
                             else () in
                     let _ = l.val := List.rev l.val in
                     let b = (l.val = []) in
                     let _ = if (not b) then while (List.hd l.val = ' ') do 
                             {
                                     l.val := List.tl l.val 
                             } 
                             else () in 
                     let _ = l.val := List.rev l.val in
                     match l.val with
                      [ [] -> []
                      | [' '::ll] -> raise Bad_Argument
                      | [x::ll] -> [x::(self#cf ll)]
                      ];
                             
                method private collapse str = 
                     let s = self#replace str in
                     let ls = self#cf (explode s) in
                     implode ls ;

                method private process_ws ws = match ws with
                  [ "preserved" -> self#preserve
                  | "replaced" -> self#replace
                  | "collapsed" -> self#collapse
                  | _ -> raise Bad_Argument
                  ];
                  
                method set_obj str = 
                     let doc = parse_wfdocument_entity default_config (from_string str) default_spec in
                     let r = doc#root in
                     let dat = (self#process_ws whitespace) r#data in
                     let b = if (or (dat = "True") (dat = "1")) then True 
                             else if (or (dat = "False") (dat = "0")) then False
                             else raise Bad_Argument in
                     v := b ;
                     
                method to_string () =  String.concat "" ["<";name;">";string_of_bool  v;"</";name;">"] ;

                method private set_attr x = match (nodename x) with
                              [ "whiteSpace" -> let r = x#attributes_as_nodes in
                                                let vv = (List.hd r)#data in
                                                let b1 = not (vv = "preserve") in
                                                let b2 = not (vv = "replace") in
                                                let b3 = not (vv = "collapse") in
                                                let b = myAnd (myAnd b1 b2) b3 in
                                                if (b) then raise Bad_Argument 
                                                else whitespace := vv 
                              | _ -> raise Bad_Attribute
                              ];

                method set_struct n sch = let alist = sch#sub_nodes in
                                          let _ = name := n in
                                          let _ = zip self#set_attr alist in
                                          ();
        end ;

