(* Copyright (c) 2002 INRIA *)
(* Author : Gaurav Chanda , Projet CRISTAL *)

open Pxp_yacc ;
open Pxp_document ;
open Pxp_types ;
open Utils ;

value string_min = 0 ;
value string_max = 32768 ;

class type string_s_sig = 
        object
             
             value mutable name : string ;
             value mutable v : string ;
             value mutable length : int ;
             value mutable minLength : int ;
             value mutable maxLength : int ;
             value mutable is_enum : bool ;
             value mutable is_fixed : bool ;
             value mutable is_fixed_minl : bool ;
             value mutable is_fixed_maxl : bool ;
             value mutable enumlist : list string ;
             value mutable whitespace : string ;

             method get_val : unit -> string ;
             method set_obj : string -> unit ;
             method to_string : unit -> string ;
             method set_struct : string ->  Pxp_document.node (Pxp_document.extension (Pxp_document.node 'a)) -> unit ;

        end ;

class string_s : string_s_sig =
        object(self)
             
             value mutable name = "Not_Named" ;
             value mutable v = "" ;
             value mutable length = -1 ;
             value mutable minLength = string_min ;
             value mutable maxLength = string_max ;
             value mutable is_enum = False ;
             value mutable is_fixed = False ;
             value mutable is_fixed_minl = False ;
             value mutable is_fixed_maxl = False ;
             value mutable enumlist = [] ;
             value mutable whitespace = "collapsed" ;

             method get_val () = v ;

             method private preserve str = str ;

             method private rf ch = match ch with
                  [ '\t' -> ' '
                  | '\n' -> ' '
                  | '\r' -> ' '
                  | c -> c
                  ];
                  
             method private replace str = implode (zip self#rf (explode str));

             method  private cf ls = 
                     let l = ref ls in
                     let b = (l.val = []) in
                     let _ = if (not b) then while (List.hd l.val = ' ') do 
                             {
                                     l.val := List.tl l.val 
                             } 
                             else () in
                     let _ = l.val := List.rev l.val in
                     let b = (l.val = []) in
                     let _ = if (not b) then while (List.hd l.val = ' ') do 
                             {
                                     l.val := List.tl l.val 
                             } 
                             else () in 
                     let _ = l.val := List.rev l.val in
                     let b = ref False in
                     match l.val with
                     [ [] -> []
                     | [' '::ll] -> if (b.val) then self#cf ll else 
                                    let _ = b.val := True in [' '::(self#cf ll)]
                     | [x::ll] -> let _ = b.val := False in [x:: (self#cf ll)]
                     ];
                             
             method private collapse str = 
                     let s = self#replace str in
                     let ls = self#cf (explode s) in
                     implode ls ;

             method private process_ws ws = match ws with
                  [ "preserved" -> self#preserve
                  | "replaced" -> self#replace
                  | "collapsed" -> self#collapse
                  | _ -> raise Bad_Argument
                  ];
                  
             method set_obj str = 
                     let doc = parse_wfdocument_entity default_config (from_string str) default_spec in
                     let r = doc#root in
                     let dat = (self#process_ws whitespace) r#data in
                     let _ = if (myAnd (length > 0) (not(String.length dat = length))) then raise Bad_Argument else () in
                     if (myAnd (String.length dat >= minLength) (String.length dat <= maxLength) ) then 
                             if (not is_enum) then v := dat
                             else if (not (is_present enumlist dat)) then raise Not_Defined_In_Enumeration 
                             else v := dat
                     else raise  Value_Not_Within_Bounds ;

             method to_string () =  String.concat "" ["<";name;">";v;"</";name;">"] ;

             method private set_attr x = match (nodename x) with
                              [ "length" -> let r = x#attributes_as_nodes in
                                            let e1 = List.hd r in
                                            let b = nodename e1 in
                                            let v1 = if (b = "value") then (int_of_string e1#data) 
                                                      else 0 in
                                            let v2 = if (b = "fixed") then (e1#data = "true")
                                                     else False in
                                            let b = List.tl r in
                                            let e2 = if (not (b = [])) then
                                                    List.hd (List.tl r) else e1 in
                                            let b = (nodename e1 = nodename e2) in
                                            let v1 = if (myAnd (not b) (nodename e2 = "value"))
                                                     then (int_of_string e2#data) else 0 in
                                            let v2 = if (myAnd (not b) (nodename e2 = "fixed"))
                                                     then ("true" = e2#data) else v2 in
                                                     if (v1 >= 0) then do {length := v1 ; is_fixed := v2 } 
                                                     else raise Bad_Argument
                              | "minLength" -> let r = x#attributes_as_nodes in
                                            let e1 = List.hd r in
                                            let b = nodename e1 in
                                            let v1 = if (b = "value") then (int_of_string e1#data) 
                                                      else 0 in
                                            let v2 = if (b = "fixed") then (e1#data = "true")
                                                     else False in
                                            let b = List.tl r in
                                            let e2 = if (not (b = [])) then
                                                    List.hd (List.tl r) else e1 in
                                            let b = (nodename e1 = nodename e2) in
                                            let v1 = if (myAnd (not b) (nodename e2 = "value"))
                                                     then (int_of_string e2#data) else 0 in
                                            let v2 = if (myAnd (not b) (nodename e2 = "fixed"))
                                                     then ("true" = e2#data) else v2 in
                                                     if (v1 <= maxLength) then
                                                             do {minLength := v1 ; is_fixed_minl := v2 } 
                                                     else raise Bad_Argument
                              | "maxLength" -> let r = x#attributes_as_nodes in
                                            let e1 = List.hd r in
                                            let b = nodename e1 in
                                            let v1 = if (b = "value") then (int_of_string e1#data) 
                                                      else 0 in
                                            let v2 = if (b = "fixed") then (e1#data = "true")
                                                     else False in
                                            let b = List.tl r in
                                            let e2 = if (not (b = [])) then
                                                    List.hd (List.tl r) else e1 in
                                            let b = (nodename e1 = nodename e2) in
                                            let v1 = if (myAnd (not b) (nodename e2 = "value"))
                                                     then (int_of_string e2#data) else 0 in
                                            let v2 = if (myAnd (not b) (nodename e2 = "fixed"))
                                                     then ("true" = e2#data) else v2 in
                                                     if (v1 >= minLength) then
                                                             do {maxLength := v1 ; is_fixed_maxl := v2 } 
                                                     else raise Bad_Argument
                              | "whiteSpace" -> let r = x#attributes_as_nodes in
                                                let vv = (List.hd r)#data in
                                                let b1 = not (vv = "preserve") in
                                                let b2 = not (vv = "replace") in
                                                let b3 = not (vv = "collapse") in
                                                let b = myAnd (myAnd b1 b2) b3 in
                                                if (b) then raise Bad_Argument 
                                                else whitespace := vv 
                              | "enumeration" -> let r = x#attributes_as_nodes in
                                                 let vv = (List.hd r)#data in
                                                 do {is_enum := True ; enumlist := [vv :: enumlist] }
                              | _ -> raise Bad_Attribute
                             ];

             method set_struct n sch = let alist = sch#sub_nodes in
                                       let _ = name := n in
                                       let _ = zip self#set_attr alist in
                                      () ;
        end ;


